#!/usr/local/bin/perl

# Try to keep the programming style good ;-)

use strict;
# Disabled for compatiblity reasons. See the according comment in
# userneu.pl
#use warnings;

# Some modules we need for password-related Stuff

use Crypt::PasswdMD5;
use Crypt::RandPasswd;

# We want to parse commandline arguments in a comfortable way

use Getopt::Std;
use Config;

# Parse the commandline arguments and fall back to default values if
# neccessary

my %opts;
getopts('b:g:u:l:S:m:x:e:D:G:hrsdy', \%opts);

if ( $opts{h} ) { usage(); } 

if ( $> != 0 ) {die "You must be root to run this.\n";}

sub usage { die "usage: $0 -u <user name;real name;class> <-g group> [-b base directory for \$HOME] [-s] [-S <smbpasswd binary>] [-r] [-m <minimum password length> [-x <maximum password length>] [-y] [-D <descriptors for the fields in input>]\n";}

if ( ! scalar %opts ) { usage(); }

my $user;
if ( $opts{u} ) { $user = $opts{u}; }
else { die "Please specify the user to be created.\n"; }

my $group = "schueler";
if ( $opts{g} ) { $group = $opts{g}; }

if ( ! getgrnam $group ) { die "Group $group doesn't exist. Please create it or specify an existing group.\n" }

my $homebase = "/home/$group";
if ( $opts{b} ) { $homebase = $opts{b}; }

my $smbpass = "/usr/local/bin/smbpasswd";
if ( $opts{S} ) { $smbpass = $opts{S}; }

my $passtype = "letters";
if ( $opts{r} ) { $passtype = "chars"; }

my @descriptor = ("name", "real name", "class");
if ( $opts{D} ) { @descriptor = split(/\;/, $opts{D}); }

my $member_groups;
if ( $opts{G} ) { $member_groups = $opts{G}; } 

my @member_groups_check;

if ( $member_groups )
	{
	@member_groups_check = split(/,/, $member_groups);

	# Now check if all the member groups specified do exist.

	membergroups_check();
	}

sub membergroups_check
{
my @failed_groups;
foreach my $membergroup ( @member_groups_check )
	{
	if ( ! getgrnam $membergroup and getgrgid 0 ne $membergroup )
		{	
		warn "$membergroup doesn't exist\n";
		push @failed_groups, $membergroup;
		}
	}
if ( @failed_groups ) { die "The following groups you specified as member groups for the users to be created do not exist: @failed_groups\n" }
}

my $member_groups_arg;
if ( $member_groups ) { $member_groups_arg = "-G" . $member_groups; }

my $minlength;
if ( $opts{m} ) { $minlength = $opts{m}; }
else	{
	if ( $opts{r} ) { $minlength = 10; }
	else { $minlength = 8; }
	}

my $maxlength;
if ( $opts{x} ) { $maxlength = $opts{x}; }
else	{
	if ( $opts{r} ) { $maxlength = 10; }
	else { $maxlength = 8; }
	}

print "Group: $group\n";
print "Base directory: $homebase\n";

# Determine the system we run on

my $os = $Config{osname};

# Do some deep magic to determine the school year.

# First get the raw time
my @time = localtime;

# Extract the relevant parts

my $month = $time[4];
my $year = $time[5];

# Convert the year to 2-digit format

my $year1 = sprintf("%02d", $year % 100);

# Now it's time for the arcane part...

my $schoolyear;
if ($month le 7) {
	my $year2=$year1 - 1;
	$schoolyear = "0$year2/$year1";
}
else {
	my $year2=$year1 + 1;
	$schoolyear = "$year1/0$year2";
}

# Make -s implicit if -S is given

if ( $opts{S} and ! $opts{s} )
	{
	$opts{s} = 1;
	}

# Check for smbpasswd(8) if -s is given

if ( $opts{s} )
	{
	if ( ! -x $smbpass ) { die "$smbpass doesn't exist or isn't executable. Please specify.\n"; }
	}

# split $user and put the results in variables

my @person = split(/\;/, $user);	

if ( scalar @person != scalar @descriptor ) { die "The number of fields in the argument to -u doesn't match the number of descriptors.\n"; }

my $name = shift @person; 

# Reduce @descriptor by one to match @person

shift @descriptor;

# Put the values of @person and @descriptor in work variables so we
# don't shift them to pieces before we're done with them.

my @person_process = @person;
my @descriptor_process = @descriptor;
my $output_rest;

while ( @person_process && @descriptor_process)
	{
	$output_rest .= (shift @descriptor_process) . ": " . (shift @person_process) . " ";
	}


# First check if the username exists. If it exists generate two
# random letters and attach them to the name. Repeat this
# procedure if the changed name exists already.

if ( $opts{d} ) { $name = dupe_engine($name); }
else { dupe_log($name,$output_rest); }

sub dupe_engine
	{
	my $username = $_[0];
	if ( getpwnam $username ) 
		{
		print "User $name already in passwd database.\n";
		until ( ! getpwnam $username )
			{
			my $randomizer = Crypt::RandPasswd->word(1,2);
			$username .= $randomizer;
			print "New Name: $username\n";
			}
		}
	return $username;
	}
sub dupe_log
	{
	my $name = $_[0];
	my $output_rest = $_[1];
	if ( getpwnam $name )
		{
		my $errlog = "duplicate_users";
		if ( $opts{e} ) { $errlog = $opts{e}; }
		print "User $name already in passwd database.\n";
		open DUPELOG,">> $errlog";
		print DUPELOG "User: $name $output_rest\n";
		close DUPELOG;
		exit 1;
		}
	}
	
# Compose the GECOS field for /etc/passwd

my $gecos;
while ( @person ) { $gecos .= (shift @person) . ";"; }

# Append the schoolyear to $gecos if -y is given

if ( $opts{y} ) { $gecos .=  $schoolyear; }

# Remove a trailing semicolon from $gecos (that just looks ugly ;-))

$gecos =~ s/\;$//;

# Generate a password for the user

my $clearpass = Crypt::RandPasswd->$passtype( $minlength, $maxlength );
if ( $opts{r} and ($clearpass =~ /'/) ) { until ( ! $clearpass =~ /'/ ) {$clearpass = Crypt::RandPasswd->$passtype($minlength, $maxlength);} }

my $passwdhash = unix_md5_crypt ($clearpass, '$1$');

# Create an Unix account
# First determine which sytem we run on and which commands we
# have to use

if ( $os eq "linux" )
	{
	# useradd(8) is braindead because it won't let me
	# specify a base directory for $HOME

	my $home = "$homebase/$name";
	if ( $home =~ /\/{2,}/ ) { $home =~ s/\/{2,}/\//g; }
	my @useradd_cmd = ("/usr/sbin/useradd", "-m", "-c$gecos", "-g$group", "-d$home");
	if ( $member_groups_arg ) { push @useradd_cmd, $member_groups_arg; }
	push @useradd_cmd, $name;
	system @useradd_cmd;
	open PASSPIPE,"| /usr/sbin/chpasswd -e";
	print PASSPIPE "$name:$passwdhash\n";
	close PASSPIPE;
	}

elsif ( $os eq "freebsd" )
	{
	my @pw_args = ("useradd", "-n $name", "-m", "-c \'$gecos\'", "-g $group", "-b $homebase", "-h 0");
	if ( $member_groups_arg ) { push @pw_args, $member_groups_arg; }
	open PWPIPE, "| /usr/sbin/pw @pw_args";
	print PWPIPE "$clearpass\n";
	close PWPIPE;
	}
else { die "Your OS is not supported.\n"; }

if ( $os eq "NetBSD" )
	{
	die "None of the NetBSD user management tools will accept the\npassword on standard input and it would have to be supplied as\ncommandline argument. If you really want to do this\nuncomment line 160 and comment line 159 I warned you.\n";
#	system "/usr/sbin/useradd -m -c '$gecos' -g $group -b $homebase -p $passwdhash $name";
	}

# Use smbpasswd(8) to create an smbpasswd entry
# Yes this is ugly...and so is smbpasswd's non-interactive
# mode...

# Compose the complete smbpasswd command

if ( $opts{s} ) { smb_create(); }

sub smb_create 
	{
	my $smbpasswd = "$smbpass -s -a $name";
	open SMBPIPE,"| $smbpasswd";
	print SMBPIPE "$clearpass\n$clearpass\n";
	close SMBPIPE;
	}

# Output the account data
print "Username: $name password: $clearpass $output_rest\n";
